#include "memory_preallocated.h"



/* Initialise the memory pool with the given amount in bytes. */

MemoryPreallocated::MemoryPreallocated(Memory *memory, length_t amount)
: _memory(memory), _amount(amount)
{
    // NOTE: Handle allocation failure.

    length_t remaining = amount;

    while (remaining)
    {
        _regions.push_back(memory->region());

        if (memory->region_size() >= remaining)
            break;

        remaining -= memory->region_size();
    }
}

/* Discard all unused regions. */

MemoryPreallocated::~MemoryPreallocated()
{
    while (!_regions.empty())
    {
        Region *region = _regions.front();

        _regions.pop_front();
        _memory->release(region);
    }
}

/* Obtain an allocated region. */

Region *MemoryPreallocated::region()
{
    std::lock_guard<std::mutex> guard(_lock);

    Region *region = _regions.front();

    _regions.pop_front();
    return region;
}

/* Release the allocated 'region'. */

void MemoryPreallocated::release(Region *region)
{
    std::lock_guard<std::mutex> guard(_lock);

    _regions.push_back(region);
}

// vim: tabstop=4 expandtab shiftwidth=4
