#pragma once

#include <systypes/base.h>

#include "types.h"



/* Region-related state information. */

class RegionState
{
public:
    unsigned long size;
    fileid_t fileid;
    offset_t filepos;

    explicit RegionState(unsigned long size=0, fileid_t fileid=0, offset_t filepos=0);

    void fill(fileid_t fileid, offset_t filepos);

    bool valid() { return size != 0; }
};



/* A memory region abstraction. */

class Region
{
public:
    address_t start, end;

    /* Debugging information. */

    RegionState state;

    /* Methods. */

    explicit Region(address_t start, address_t end);

    virtual ~Region();

    length_t size();

    int compare(Region *other);

    void fill(fileid_t fileid, offset_t filepos);

    void flush();

    /* Simulation methods. */

    char *read(length_t offset=0);

    void write(const char *data, length_t offset=0);
};

// vim: tabstop=4 expandtab shiftwidth=4
